# encoding: utf-8
import es, gamethread, effectlib

import psyco
psyco.full()

beacon_users = {}

sas_beacon_size = es.ServerVar('sas_beacon_size', 0, "sas : Rayon du balisage")
sas_beacon_r = es.ServerVar('sas_beacon_r', 0, "sas : Couleur Rouge, 0 à 255")
sas_beacon_v = es.ServerVar('sas_beacon_v', 0, "sas : Couleur Vert, 0 à 255")
sas_beacon_b = es.ServerVar('sas_beacon_b', 0, "sas : Couleur Bleu, 0 à 255")
sas_beacon_team = es.ServerVar('sas_beacon_team', 0, "sas : La couleur du balisage et de la couleur de la Team du joueur meme")

class Beacon:
    def __init__(self, userid):
        self.repeat = 0
        self.delay = 0
        self.userid = int(userid)
        self.sound = "sas/sas_beacon.wav"
        self.model = "materials/sprites/lgtning.vmt"
        
        if sas_beacon_team == 1:
            if es.getplayerteam(self.userid) == 2:
                self.color = "255 0 0 255"
            if es.getplayerteam(self.userid) == 3:
                self.color = "0 0 255 255"
        else:
            self.color = "%d %d %d 255" % (int(sas_beacon_r), int(sas_beacon_v), int(sas_beacon_b))
        self.Type = "Normal"
        self.Function = None
       
    def Start(self, delay=1, repeat = "INDEFINITELY"):
        self.delay = delay
        self.repeat = repeat
        gamethread.delayedname(delay, '%d_beacon' % self.userid, self.Loop)
   
    def Stop(self, delete = False):
        self.repeat = 0
        self.delay = 0
        gamethread.cancelDelayed('%d_beacon' % self.userid)
        Delete(self.userid)
       
    def Loop(self):
        if not self.repeat == "INDEFINITELY" and self.repeat > 0:
            self.repeat -= 1
        elif not self.repeat == "INDEFINITELY":
            self.Stop(True)
            return
        if not es.getplayerprop(self.userid, 'CBasePlayer.pl.deadflag'):
            if self.Type == "Easy":
                r, g, b, a = str(self.color).split(' ')
                location = es.getplayerlocation(self.userid)
                effectlib.drawCircle(location, 150, steps=50, model=self.model, seconds=0.2, width=20, endwidth=10, red=r, green=g, blue=b, brightness = a, speed=70)
                if self.sound:
                    es.emitsound('player', self.userid, self.sound, 1.0, 0.7)
            elif self.Type == "Normal":
                if callable(self.Function):
                    self.Function(self.userid)
                else:
                    es.doblock(self.Function)
            gamethread.delayedname(self.delay, '%d_beacon' % self.userid, self.Loop)
        else:
            self.Stop(True)
           
    def Delete(self):
        self.Stop(True)

def EasyBeacon(userid):
    userid = int(userid)
    if not beacon_users.has_key(userid):
        beacon_users[userid] = Beacon(userid)
        beacon_users[userid].Type = "Easy"
    return beacon_users[userid]
   
def Delete(userid):
    userid = int(userid)
    if not beacon_users.has_key(userid):
        es.dbgmsg(0, 'ERROR: Cannot find instance %s inside beacon'%userid)
    else:
        gamethread.cancelDelayed('%d_beacon' % userid)
        del beacon_users[userid]

selfmodule = __import__('beaconlib')

def player_disconnect(ev):
    userid = ev['userid']
    if beacon_users.has_key(userid):
        gamethread.cancelDelayed('%d_beacon' % userid)
        Delete(userid)
es.addons.registerForEvent(selfmodule, 'player_disconnect', player_disconnect)
       
def player_death(ev):
    userid = ev['userid']
    if beacon_users.has_key(userid):
        gamethread.cancelDelayed('%d_beacon' % userid)
        Delete(userid)
es.addons.registerForEvent(selfmodule, 'player_death', player_death)
       
def round_end(ev):
    for userid in es.getUseridList():
        if not es.getplayerprop(userid, 'CBasePlayer.pl.deadflag'):
            userid = int(userid)
            if beacon_users.has_key(userid):
                Delete(userid)
es.addons.registerForEvent(selfmodule, 'round_end', round_end)

def server_cvar(event_var):
    if event_var['cvarname'] == 'mp_restartgame' and int(event_var['cvarvalue']) == 1:
        for userid in beacon_users.keys():
            gamethread.cancelDelayed('%d_beacon' % userid)
            Delete(userid)
es.addons.registerForEvent(selfmodule, 'server_cvar', server_cvar)

def round_start(event_var):
    for userid in beacon_users.keys():
        gamethread.cancelDelayed('%d_beacon' % userid)
        Delete(userid)
es.addons.registerForEvent(selfmodule, 'round_start', round_start)

